@@ -97,13 +97,23 @@ class RequestHandler(postgres.RequestHandlerMixin,
     def on_finish(self) -> None:
         """Invoked after a request has completed"""
         super().on_finish()
-        metric_id = '{}.{}'.format(self.NAME, self.request.method)
         self.application.loop.add_callback(
             self.application.stats.incr,
-            'response.{}.{}'.format(metric_id, self.get_status()))
+            {
+                'key': 'http_requests',
+                'endpoint': self.NAME,
+                'method': self.request.method,
+                'status': self.get_status()
+            })
+
         self.application.loop.add_callback(
             self.application.stats.add_duration,
-            'request.{}.{}'.format(metric_id, self.get_status()),
+            {
+                'key': 'http_request_duration',
+                'endpoint': self.NAME,
+                'method': self.request.method,
+                'status': self.get_status()
+            },
             self.request.request_time())
 
     def compute_etag(self) -> None:
@@ -133,6 +143,19 @@ class RequestHandler(postgres.RequestHandlerMixin,
         namespace.update({'version': version})
         return namespace
 
+    def on_postgres_timing(self,
+                           metric_name: str,
+                           duration: float) -> None:
+        """Invoked by sprockets-postgres after each query"""
+        self.application.loop.add_callback(
+            self.application.stats.add_duration,
+            {
+                'key': 'postgres_query_duration',
+                'query': metric_name,
+                'endpoint': self.NAME
+            },
+            duration)
+
     def send_response(self, value: typing.Union[dict, list]) -> None:
         """Send the response to the client"""
         if 'self' not in self._links:
@@ -182,14 +205,6 @@ class RequestHandler(postgres.RequestHandlerMixin,
         """Adds the cache response headers for the object being returned."""
         self.add_header('Cache-Control', 'public, max-age={}'.format(ttl))
 
-    def _on_postgres_timing(self,
-                            metric_name: str,
-                            duration: float) -> None:
-        """Invoked by sprockets-postgres after each query"""
-        self.application.loop.add_callback(
-            self.application.stats.add_duration,
-            'postgres.{}'.format(metric_name), duration)
-
     @property
     def _respond_with_html(self) -> bool:
         """Returns True if the current response should respond with HTML"""
@@ -350,7 +365,8 @@ class CRUDRequestHandler(ValidatingRequestHandler):
         original = dict(result.row)
         for key in {'created_at', 'created_by',
                     'last_modified_at', 'last_modified_by'}:
-            del original[key]
+            if key in original:
+                del original[key]
 
         for key, value in original.items():
             if isinstance(value, uuid.UUID):
@@ -384,7 +400,7 @@ class CRUDRequestHandler(ValidatingRequestHandler):
         # Send the new record as a response
         await self._get(self._get_query_kwargs(updated))
 
-    async def _post(self, kwargs) -> None:
+    async def _post(self, kwargs) -> dict:
         values = self.get_request_body()
 
         # Handle compound keys for child object CRUD
@@ -409,6 +425,7 @@ class CRUDRequestHandler(ValidatingRequestHandler):
 
             # Return the record as if it were a GET
         await self._get(self._get_query_kwargs(result.row))
+        return result.row
 
 
 class CollectionRequestHandler(CRUDRequestHandler):
