@@ -1,13 +1,16 @@
+from __future__ import annotations
+
 import re
 import typing
 
 import psycopg2.errors
 
 from imbi import common, errors
-from imbi.endpoints import base
+from imbi.endpoints import base, projects
 
 
-class CollectionRequestHandler(base.CollectionRequestHandler):
+class CollectionRequestHandler(projects.OpensearchMixin,
+                               base.CollectionRequestHandler):
 
     NAME = 'project-fact-types'
     ID_KEY = 'project_id'
@@ -35,6 +38,8 @@ class CollectionRequestHandler(base.CollectionRequestHandler):
                                            WHERE fact_type_id = b.fact_type_id
                                              AND b.value::NUMERIC(9,2)
                                          BETWEEN min_value AND max_value)
+                              WHEN a.data_type = 'boolean' AND b.value = 'true'
+                              THEN 100
                               ELSE 0
                           END
                 END AS score,
@@ -63,15 +68,46 @@ class CollectionRequestHandler(base.CollectionRequestHandler):
         self.send_response(common.coerce_project_fact_values(result.rows))
 
     async def post(self, *args, **kwargs):
-        for fact in self.get_request_body():
+        facts = await self._coerce_facts(self.get_request_body())
+        for fact in facts:
             fact.update({
                 'project_id': kwargs['project_id'],
                 'username': self._current_user.username
             })
             await self.postgres_execute(
                 self.POST_SQL, fact, 'post-{}'.format(self.NAME))
+        await self.index_document(kwargs['project_id'])
         self.set_status(204)
 
+    async def _coerce_facts(self, facts: list[dict]) -> list[dict]:
+        """Take a list of project facts and coerce their values.
+
+        This method looks up the data types and then calls
+        :func:`common.coerce_project_fact_values` to perform
+        the coercion appropriately.
+
+        """
+        result = await self.postgres_execute(
+            'SELECT id, data_type'
+            '  FROM v1.project_fact_types'
+            ' WHERE id IN %(fact_type_ids)s',
+            {'fact_type_ids': tuple(fact['fact_type_id'] for fact in facts)},
+        )
+        fact_type_map = {row['id']: row['data_type'] for row in result.rows}
+        for fact in facts:
+            try:
+                fact_type = fact_type_map[fact['fact_type_id']]
+            except KeyError:
+                raise errors.BadRequest('%s is not a known fact type id',
+                                        fact['fact_type_id'])
+            else:
+                fact['data_type'] = fact_type
+
+        try:
+            return common.coerce_project_fact_values(facts)
+        except ValueError as error:
+            raise errors.BadRequest('invalid fact value: %s', error)
+
     def on_postgres_error(self,
                           metric_name: str,
                           exc: Exception) -> typing.Optional[Exception]:
