@@ -1,10 +1,20 @@
 import asyncio
+import dataclasses
 import re
+import typing
 
-from imbi import errors
+from imbi import errors, models
 from imbi.endpoints import base
 
 
+def project_to_dict(project: models.Project) -> dict:
+    output = dataclasses.asdict(project)
+    for key in ['namespace', 'project_type']:
+        output[f'{key}_slug'] = output[key]['slug']
+        output[key] = output[key]['name']
+    return output
+
+
 class _RequestHandlerMixin:
 
     ITEM_NAME = 'project'
@@ -40,7 +50,41 @@ class _RequestHandlerMixin:
          WHERE a.id=%(id)s""")
 
 
-class CollectionRequestHandler(_RequestHandlerMixin,
+class OpensearchMixin:
+
+    SEARCH_INDEX = 'projects'
+
+    async def delete_from_index(self,
+                                project_id: typing.Union[int, str]) -> None:
+        await self.application.opensearch.delete_document(
+            self.SEARCH_INDEX, str(project_id))
+
+    async def index_document(self, project_id: typing.Union[int, str]) -> None:
+        project = await models.project(project_id, self.application)
+        await self.application.opensearch.index_document(
+            self.SEARCH_INDEX, str(project_id), project_to_dict(project))
+
+
+class ProjectAttributeCollectionMixin(OpensearchMixin):
+
+    async def post(self, *_args, **kwargs):
+        result = await self._post(kwargs)
+        await self.index_document(result['project_id'])
+
+
+class ProjectAttributeCRUDMixin(OpensearchMixin):
+
+    async def delete(self, *args, **kwargs):
+        await super().delete(*args, **kwargs)
+        await self.index_document(kwargs['project_id'])
+
+    async def patch(self, *args, **kwargs):
+        await super().patch(*args, **kwargs)
+        await self.index_document(kwargs['project_id'])
+
+
+class CollectionRequestHandler(OpensearchMixin,
+                               _RequestHandlerMixin,
                                base.CollectionRequestHandler):
     NAME = 'projects'
     IS_COLLECTION = True
@@ -141,8 +185,14 @@ class CollectionRequestHandler(_RequestHandlerMixin,
             'rows': count.row['records'],
             'data': result.rows})
 
+    async def post(self, *_args, **kwargs):
+        result = await self._post(kwargs)
+        await self.index_document(result['id'])
 
-class RecordRequestHandler(_RequestHandlerMixin, base.CRUDRequestHandler):
+
+class RecordRequestHandler(OpensearchMixin,
+                           _RequestHandlerMixin,
+                           base.CRUDRequestHandler):
 
     NAME = 'project'
 
@@ -185,7 +235,16 @@ class RecordRequestHandler(_RequestHandlerMixin, base.CRUDRequestHandler):
                a.name,
                b.recorded_at,
                b.recorded_by,
-               b.value,
+               CASE WHEN a.data_type = 'boolean' THEN b.value::bool::text
+                    WHEN a.data_type = 'date' THEN b.value::date::text
+                    WHEN a.data_type = 'decimal'
+                         THEN b.value::numeric(9,2)::text
+                    WHEN a.data_type = 'integer'
+                         THEN b.value::integer::text
+                    WHEN a.data_type = 'timestamp'
+                         THEN b.value::timestamptz::text
+                    ELSE b.value
+                END AS value,
                a.data_type,
                a.fact_type,
                a.ui_options,
@@ -201,6 +260,8 @@ class RecordRequestHandler(_RequestHandlerMixin, base.CRUDRequestHandler):
                                            WHERE fact_type_id = b.fact_type_id
                                              AND b.value::NUMERIC(9,2)
                                          BETWEEN min_value AND max_value)
+                              WHEN a.data_type = 'boolean'
+                               AND b.value = 'true' THEN 100
                               ELSE 0
                           END
                 END AS score,
@@ -251,6 +312,10 @@ class RecordRequestHandler(_RequestHandlerMixin, base.CRUDRequestHandler):
                pagerduty_service_id=%(pagerduty_service_id)s
          WHERE id=%(id)s""")
 
+    async def delete(self, *args, **kwargs):
+        await super().delete(*args, **kwargs)
+        await self.delete_from_index(kwargs['id'])
+
     async def get(self, *args, **kwargs):
         if self.get_argument('full', 'false') == 'true':
             query_args = self._get_query_kwargs(kwargs)
@@ -276,3 +341,41 @@ class RecordRequestHandler(_RequestHandlerMixin, base.CRUDRequestHandler):
             self.send_response(output)
         else:
             await self._get(kwargs)
+
+    async def patch(self, *args, **kwargs):
+        await super().patch(*args, **kwargs)
+        await self.index_document(kwargs['id'])
+
+
+class SearchRequestHandler(base.AuthenticatedRequestHandler):
+
+    async def get(self):
+        result = await self.application.opensearch.client.search(
+            body={
+                'query': {
+                    'query_string': {
+                        'query': self.get_query_argument('s')}},
+                'size': 1000},
+            index='projects')
+        self.send_response({
+            'hits': [r['_source'] for r in result['hits']['hits']]})
+
+
+class SearchIndexRequestHandler(base.AuthenticatedRequestHandler):
+
+    SQL = re.sub(r'\s+', ' ', """\
+        SELECT id
+          FROM v1.projects
+         ORDER BY id""")
+
+    async def get(self):
+        result = await self.postgres_execute(self.SQL)
+        ids = [row['id'] for row in result]
+        for project_id in ids:
+            project = await models.project(project_id, self.application)
+            await self.application.opensearch.index_document(
+                'projects', project_id, project_to_dict(project))
+
+        self.send_response({
+            'status': 'ok',
+            'message': f'Queued {len(ids)} projects for indexing'})
